@@ -1,15 +1,15 @@
 from __future__ import annotations
 import enum
 
-from typing import Tuple, Deque, List
+from typing import Tuple, Deque, List, Final
 from spacepackets.log import get_console_logger
 
-SPACE_PACKET_HEADER_SIZE = 6
+SPACE_PACKET_HEADER_SIZE: Final = 6
 
 
 class PacketTypes(enum.IntEnum):
-    PACKET_TYPE_TM = 0
-    PACKET_TYPE_TC = 1
+    TM = 0
+    TC = 1
 
 
 class SequenceFlags(enum.IntEnum):
@@ -27,7 +27,38 @@ class SpacePacketHeader:
             packet_version: int = 0b000, secondary_header_flag: bool = True,
             sequence_flags: SequenceFlags = SequenceFlags.UNSEGMENTED
     ):
+        """Create a space packet header with the given field parameters
+
+        :param packet_type: 0 for Telemetery, 1 for Telecommands
+        :param apid: Application Process ID, should not be larger
+            than 11 bits, deciaml 2074 or hex 0x7ff
+        :param source_sequence_count: Sequence counter, should not be larger than 0x3fff or
+            decimal 16383
+        :param data_length: Should not be largen than 65535 bytes
+        :param packet_version:
+        :param secondary_header_flag:
+        :param sequence_flags:
+        :raises ValueError: On invalid parameters
+        """
         self.packet_type = packet_type
+        if apid > pow(2, 11) - 1 or apid < 0:
+            logger = get_console_logger()
+            logger.warning(
+                f'Invalid APID, exceeds maximum value {pow(2, 11) - 1} or negative'
+            )
+            raise ValueError
+        if source_sequence_count > pow(2, 14) - 1 or source_sequence_count < 0:
+            logger = get_console_logger()
+            logger.warning(
+                f'Invalid source sequence count, exceeds maximum value {pow(2, 14)- 1} or negative'
+            )
+            raise ValueError
+        if data_length > pow(2, 16) - 1 or data_length < 0:
+            logger = get_console_logger()
+            logger.warning(
+                f'Invalid data length value, exceeds maximum value of {pow(2, 16) - 1} or negative'
+            )
+            raise ValueError
         self.apid = apid
         self.ssc = source_sequence_count
         self.secondary_header_flag = secondary_header_flag
@@ -42,17 +73,8 @@ class SpacePacketHeader:
             apid=self.apid
         )
 
-    @classmethod
-    def __empty(cls) -> SpacePacketHeader:
-        return SpacePacketHeader(
-            packet_type=PacketTypes.PACKET_TYPE_TC,
-            apid=0,
-            source_sequence_count=0,
-            data_length=0
-        )
-
     def pack(self) -> bytearray:
-        """Serialize raw space packet header.
+        """Serialize raw space packet header into a bytearray
         """
         header = bytearray()
         header.append((self.packet_id & 0xff00) >> 8)
@@ -63,19 +85,33 @@ class SpacePacketHeader:
         header.append(self.data_length & 0xff)
         return header
 
+    @property
+    def header_length(self) -> int:
+        return SPACE_PACKET_HEADER_SIZE
+
+    @property
+    def packet_length(self) -> int:
+        """Retrieve the full space packet size when packed
+        :return: Size of the TM packet based on the space packet header data length field.
+        The space packet data field is the full length of data field minus one without
+        the space packet header.
+        """
+        return SPACE_PACKET_HEADER_SIZE + self.data_length + 1
+
     @classmethod
     def unpack(cls, space_packet_raw: bytes) -> SpacePacketHeader:
-        """Unpack a raw space packet into the space packet header
-        :raise ValueError: Raw packet length invalid"""
+        """Unpack a raw space packet into the space packet header instance
+        :raise ValueError: Raw packet length invalid
+        """
         if len(space_packet_raw) < SPACE_PACKET_HEADER_SIZE:
             logger = get_console_logger()
             logger.warning('Packet size smaller than PUS header size!')
             raise ValueError
         packet_type = space_packet_raw[0] & 0x10
         if packet_type == 0:
-            packet_type = PacketTypes.PACKET_TYPE_TM
+            packet_type = PacketTypes.TM
         else:
-            packet_type = PacketTypes.PACKET_TYPE_TC
+            packet_type = PacketTypes.TC
         packet_version = space_packet_raw[0] >> 5
         secondary_header_flag = (space_packet_raw[0] & 0x8) >> 3
         apid = ((space_packet_raw[0] & 0x7) << 8) | space_packet_raw[1]
@@ -92,14 +128,13 @@ class SpacePacketHeader:
             source_sequence_count=ssc
         )
 
-    def append_space_packet_header_content(self, content_list: list):
-        content_list.append(str(hex(self.apid)))
-        content_list.append(str(self.ssc))
+    def __str__(self):
+        return f"Space Packet with Packet ID 0x{self.packet_id:04x}, APID {self.apid}, " \
+               f"SSC {self.ssc}, Data Length {self.data_length}"
 
-    @staticmethod
-    def append_space_packet_header_column_headers(header_list: list):
-        header_list.append("APID")
-        header_list.append("SSC")
+    def __repr__(self):
+        return f"{self.__class__.__name__}(packet_type={self.packet_type!r}, " \
+               f"packet_id={self.packet_id!r}, apid={self.apid!r}, ssc={self.ssc!r})"
 
 
 def get_space_packet_id_bytes(
@@ -108,9 +143,9 @@ def get_space_packet_id_bytes(
     """This function also includes the first three bits reserved for the version.
 
     :param version: Version field of the packet ID. Defined to be 0b000 in the space packet standard
-    :param packet_type:
+    :param packet_type: 0 for TM, 1 for TC
     :param secondary_header_flag: Indicates presence of absence of a Secondary Header
-    in the Space Packet
+        in the Space Packet
     :param apid: Application Process Identifier. Naming mechanism for managed data path, has 11 bits
     :return:
     """
@@ -131,27 +166,24 @@ def get_space_packet_id_num(
 def get_space_packet_sequence_control(
         sequence_flags: SequenceFlags, source_sequence_count: int
 ) -> int:
-    """Get sequence control in integer format
-    """
-    if sequence_flags > 3:
+    """Get sequence control in integer format"""
+    if sequence_flags > SequenceFlags.UNSEGMENTED:
         logger = get_console_logger()
-        logger.warning(
-            "get_sp_packet_sequence_control: Sequence flag value larger than 0b11! "
-            "Setting to 0b11.."
-        )
-        sequence_flags = SequenceFlags.UNSEGMENTED
+        logger.warning('Sequence flag value larger than 0b11')
+        raise ValueError
     if source_sequence_count > 0x3fff:
         logger = get_console_logger()
         logger.warning(
-            "get_sp_packet_sequence_control: Source sequence count largen than 0x3fff. "
-            "Larger bits are cut off!"
+            'get_sp_packet_sequence_control: Source sequence count largen than 0x3fff'
         )
-    return (source_sequence_count & 0x3FFF) | (sequence_flags << 14)
+        raise ValueError
+    return (source_sequence_count & 0x3fff) | (sequence_flags << 14)
 
 
 def get_space_packet_header(
         packet_id: int, packet_sequence_control: int, data_length: int
 ) -> bytearray:
+    """Retrieve raw space packet header from the three required values"""
     header = bytearray()
     header.append((packet_id & 0xff00) >> 8)
     header.append(packet_id & 0xff)
@@ -162,23 +194,15 @@ def get_space_packet_header(
     return header
 
 
-def get_space_packet_header_from_packet_id_bytes(
-        packet_id_byte_one: int, packet_id_byte_two: int, packet_sequence_control: int,
-        data_length: int
-) -> bytearray:
-    header = bytearray()
-    header.append(packet_id_byte_one)
-    header.append(packet_id_byte_two)
-    header.append((packet_sequence_control & 0xFF00) >> 8)
-    header.append(packet_sequence_control & 0xFF)
-    header.append((data_length & 0xFF00) >> 8)
-    header.append(data_length & 0xFF)
-    return header
-
-
 def get_apid_from_raw_space_packet(raw_packet: bytes) -> int:
+    """Retrieve the APID from the raw packet.
+
+    :param raw_packet:
+    :raises ValueError: Passed bytearray too short
+    :return:
+    """
     if len(raw_packet) < 6:
-        return 0
+        raise ValueError
     return ((raw_packet[0] & 0x7) << 8) | raw_packet[1]
 
 
@@ -190,13 +214,13 @@ def get_total_space_packet_len_from_len_field(len_field: int):
 
 
 def parse_space_packets(
-        analysis_queue: Deque[bytearray], packet_id: int, max_len: int
+        analysis_queue: Deque[bytearray], packet_ids: Tuple[int]
 ) -> List[bytearray]:
     """Given a deque of bytearrays, parse for space packets. Any broken headers will be removed.
+    If a packet is detected and the
     Any broken tail packets will be reinserted into the given deque
     :param analysis_queue:
-    :param packet_id:
-    :param max_len:     Maximum allowed packet length
+    :param packet_ids:
     :return:
     """
     tm_list = []
@@ -211,22 +235,14 @@ def parse_space_packets(
         return tm_list
     # Packet ID detected
     while True:
-        if current_idx > max_len - 6 or current_idx > len(concatenated_packets) - 1:
+        if current_idx + 1 >= len(concatenated_packets):
             break
-        try:
-            current_packet_id = \
-                (concatenated_packets[current_idx] << 8) | concatenated_packets[current_idx + 1]
-        except IndexError:
-            logger = get_console_logger()
-            logger.warning(
-                f'Index error, current index {current_idx} larger than length of concatenated '
-                f'packets {len(concatenated_packets)}'
-            )
-            return tm_list
-        if current_packet_id == packet_id:
+        current_packet_id = \
+            (concatenated_packets[current_idx] << 8) | concatenated_packets[current_idx + 1]
+        if current_packet_id in packet_ids:
             result, current_idx = __handle_packet_id_match(
                 concatenated_packets=concatenated_packets, analysis_queue=analysis_queue,
-                max_len=max_len, current_idx=current_idx, tm_list=tm_list
+                current_idx=current_idx, tm_list=tm_list
             )
             if result != 0:
                 break
@@ -237,21 +253,14 @@ def parse_space_packets(
 
 
 def __handle_packet_id_match(
-        concatenated_packets: bytearray, analysis_queue: Deque[bytearray], max_len: int,
+        concatenated_packets: bytearray, analysis_queue: Deque[bytearray],
         current_idx: int, tm_list: List[bytearray]
 ) -> (int, int):
     next_packet_len_field = \
         concatenated_packets[current_idx + 4] | concatenated_packets[current_idx + 5]
     total_packet_len = get_total_space_packet_len_from_len_field(next_packet_len_field)
-    if total_packet_len > max_len:
-        print(
-            f'parse_space_packets: Detected packet length larger than specified maximum'
-            f'length {max_len}. Skipping header..'
-        )
-        # Packet too long. Throw away the header and advance index
-        current_idx += 6
     # Might be part of packet. Put back into analysis queue as whole
-    elif total_packet_len > len(concatenated_packets):
+    if total_packet_len > len(concatenated_packets):
         analysis_queue.appendleft(concatenated_packets)
         return -1, current_idx
     else:
